#include <QtGui>
#include "rowedlist.h"

RowedList::RowedList(const QFont &font,bool b,QWidget *parent)
        : QWidget(parent),curFont(font),  showMissing(b)
{
    setMouseTracking(true);
    oldChar=QChar('6');
    curHeaderFont=parent->font();

    //    
    QFontMetrics fm(curFont);
    curRowHeight=fm.height()+3;
    curCellWidth=curRowHeight;
    curHeaderWidth=0;
    numberOfCols=(int)(width() / curCellWidth);
    numberOfRows=(int)(height()/curRowHeight);
    curCount=0;
    lastKey = qMakePair(-1,-1);
    fullItems.clear();
    newItems.clear();
    QSizePolicy policy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    policy.setHeightForWidth(true);
    setSizePolicy(policy);
}

bool RowedList::event(QEvent *event)
{
    if (event->type()==QEvent::MouseMove) {
        QMouseEvent *e=static_cast<QMouseEvent *>(event);
        if (!e) return QWidget::event(event);
        int y=e->y();
        int x=e->x();
        if (e->button() == Qt::NoButton) {
            if (x<=curHeaderWidth+1 || x>=curHeaderWidth+numberOfCols*curCellWidth ||
                    y>=numberOfRows*curRowHeight || curCount==0)
                    return QWidget::event(event);
            int i=y/curRowHeight;
            int i1=getActualRow(i);
            int j=(x-curHeaderWidth)/curCellWidth;
            QStringList list=newItems.at(i1).split("|");
            int j1=list.at(1).indexOf(items.at(i))+j;
            if (j1<list.at(1).length()) {
                if (oldChar!=list.at(1).at(j1)) {
                    emit characterHovered(list.at(1).at(j1));
                    oldChar=list.at(1).at(j1);
                }
            }
        }
    }
    return QWidget::event(event);
}

int RowedList::heightForWidth( int /*width */) const
{
    return curRowHeight*numberOfRows+3;
}


QSize RowedList::sizeHint() const
{
    int w = curHeaderWidth+3+numberOfCols*curCellWidth;
    return QSize(w,heightForWidth(w));
}

void RowedList::setShowMissingCharacters(bool showMissing)
{
    this->showMissing=showMissing;
    setItems(fullItems);
}

void RowedList::clear()
{
    QStringList l;
    setItems(l);
    lastKey = qMakePair(-1,-1);
}

void RowedList::res(int i)
{
    numberOfCols=(i-curHeaderWidth) / curCellWidth;
    headers.clear();
    items.clear();
    if (curCount>0) {
        int co=newItems.count();
        for (int i=0; i<co; i++) {
            QStringList list=newItems.at(i).split("|");
            QString header=list.at(0)+"|";
            QString s=list.at(1);
            headers.append(header);
            items.append(s.left(numberOfCols));
            s.remove(0,numberOfCols);
            while (s.length()>0) {
                headers.append("");
                items.append(s.left(numberOfCols));
                s.remove(0,numberOfCols);
            }
        }
        numberOfRows=items.count();
    }
    else numberOfRows=1;
}

void RowedList::resizeEvent(QResizeEvent *event)
{
    res(event->size().width());
    update();
}

void RowedList::setRowHeight(int newRowHeight)
{
    curRowHeight=newRowHeight;
    adjustSize();
    update();
    updateGeometry();
}

void RowedList::setCellWidth(int newCellWidth)
{
    curCellWidth=newCellWidth;
    adjustSize();
    update();
    updateGeometry();
}

void RowedList::setHeaderWidth(int newHeaderWidth)
{
    curHeaderWidth=newHeaderWidth;
    adjustSize();
    update();
    updateGeometry();
}

//  - ,   
void RowedList::setFont(const QFont &newFont)
{
    curFont=newFont;
    QFontMetrics fm(curFont);
    curRowHeight=fm.height()+3;
    curCellWidth=curRowHeight;
    adjustSize();
    update();
    updateGeometry();
}

void RowedList::setItems(const QStringList &list)
{
    if (fullItems!=list) fullItems=list;
    newItems.clear();
    curCount=list.count();

    if (curCount>0) {
        QFontMetrics fm(curHeaderFont);
        QFontMetrics fm1(curFont);
        //   
        int w=0;
        int curw=0;

        for (int i=0; i<curCount; i++) {
            QStringList l=list.at(i).split("|");
            QString s=l.at(1);
            QString s1=s;
            if (!showMissing) {
                for (int j=0; j<s.length(); j++)
                if (!fm1.inFont(s.at(j))) s1.remove(s.at(j));
            }
            if (s1.length()!=0) {
                newItems.append(l.at(0)+"|"+s1);
                curw=fm.width(l.at(0));
                if (curw>w) w=curw;
            }
        }
        curHeaderWidth=w>0?w+5:0;
    }
    curCount=newItems.count();
    lastKey = qMakePair(-1,-1);
    /*resize(width()+1,heightForWidth(width()));
    resize(curHeaderWidth+3+numberOfCols*curCellWidth, heightForWidth(curHeaderWidth+3+numberOfCols*curCellWidth));*/

    update();
    adjustSize();
    updateGeometry();
}

void RowedList::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    QPen pen;
    QPen pen1(Qt::gray, 1, Qt::DotLine, Qt::SquareCap, Qt::BevelJoin);
    QVector<qreal> dashes;
    dashes << 3.0 << 1.0;
    pen1.setDashPattern(dashes);
    painter.setPen(Qt::white);
    painter.setBrush(Qt::white);
    painter.drawRect(0,0,width(),height());
    if (curCount>0) {
        //   
        QRect redrawRect = event->rect();
        int beginRow = redrawRect.top()/curRowHeight;
        int endRow = qMin(redrawRect.bottom()/curRowHeight,items.count()-1);

        // ,  
        if (curHeaderWidth>2) {
            painter.setPen(pen1);
            painter.drawLine(curHeaderWidth,redrawRect.top(),
            curHeaderWidth,redrawRect.bottom());
        }

        QFontMetrics fm(curFont);
        //
        painter.setFont(curHeaderFont);
        QString s;
        for (int i=beginRow;i<=endRow;i++) {
            if (!headers.at(i).isEmpty()) {
                s=headers.at(i);
                painter.setPen(pen1);
                if (i>0)
                painter.drawLine(0,i*curRowHeight,
                 curHeaderWidth+numberOfCols*curCellWidth,i*curRowHeight);
            } else {
                if (i==beginRow) {
                    int j=i;
                    while ((j>=0) && (headers.at(j).isEmpty())) j--;
                    s=headers.at(qMax(j,0));
                }
            }
            painter.setPen(pen);
            painter.drawText(QRect(0,i*curRowHeight,curHeaderWidth,curRowHeight),
                             Qt::AlignCenter, s.left(s.length()-1));
        }
        //
        painter.setFont(curFont);
        painter.setPen(pen);
        for (int i=beginRow;i<=endRow;i++) {
            s=items.at(i);
            int i1=getActualRow(i);
            for (int j=0; j<s.length(); ++j) {
               if (lastKey.first==i1 && lastKey.second==getActualCol(i,j,i1))
                    painter.fillRect(curHeaderWidth+j*curCellWidth+1,i*curRowHeight+1,
                        curCellWidth-1,curRowHeight-1, QBrush(Qt::lightGray));
                QChar c=s.at(j);
                if (!fm.inFont(c)) c=QChar(QChar::ReplacementCharacter);
                painter.drawText(QRect(curHeaderWidth+j*curCellWidth,
                        i*curRowHeight+2,curCellWidth,curRowHeight),Qt::AlignCenter, c);
            }
        }
    }
}

void RowedList::mousePressEvent(QMouseEvent *event)
{
    int y=event->y();
    int x=event->x();
    if (event->button() == Qt::LeftButton) {
        if (x<=curHeaderWidth+1 || x>=curHeaderWidth+numberOfCols*curCellWidth ||
                y>=numberOfRows*curRowHeight || curCount==0) return;

        int i=y/curRowHeight;
        int i1=getActualRow(i);
        int j=(x-curHeaderWidth)/curCellWidth;
        QStringList list=newItems.at(i1).split("|");
        int j1=list.at(1).indexOf(items.at(i))+j;
        if (j1<list.at(1).length()) {
            lastKey=qMakePair(i1,j1);
            emit characterSelected(QString(list.at(1).at(j1)));
        }
        update();
    }
    else {
        QWidget::mousePressEvent(event);
    }
}

int RowedList::getActualRow(int i)
{
    int i1=0;
    for (int qw=0;qw<=i;++qw)
            if (!headers.at(qw).isEmpty()) i1+=1;
    return --i1;
}

int RowedList::getActualCol(int i,int j,int i1)
{
    QStringList list=newItems.at(i1).split("|");
    return list.at(1).indexOf(items.at(i))+j;
}
